--=============================================================================
--=============================================================================
library IEEE;
use IEEE.electrical_systems.all;
--=============================================================================
  entity IBIS_R is

    generic (Rval  : real := 1.0;
             Scale : real := 1.0);

    port (terminal p, n : electrical);

  end entity IBIS_R;
  --===========================================================================
  architecture ideal of IBIS_R is

  quantity Vout across Iout through p to n;

  begin

    Vout == Scale * Rval * Iout;

  end architecture ideal;
--=============================================================================
--=============================================================================
library IEEE;
use IEEE.electrical_systems.all;
--=============================================================================
  entity IBIS_L is

    generic (Lval  : real := 1.0;
             I0    : real := 0.0;
             Scale : real := 1.0);

    port (terminal p, n : electrical);

  end entity IBIS_L;
  --===========================================================================
  architecture ideal of IBIS_L is

  quantity Vout across Iout through p to n;

  begin

    if domain = quiescent_domain use
      Iout == I0;
    else
      Vout == Scale * Lval * Iout'dot;
    end use;

  end architecture ideal;
--=============================================================================
--=============================================================================
library IEEE;
use IEEE.electrical_systems.all;
--=============================================================================
  entity IBIS_T is

    generic (Z0 : real := 50.0;
             TD : real := 1.0e-9);

    port (terminal n1, ref1, n2, ref2 : electrical);

  end entity IBIS_T;
  --===========================================================================
  architecture Ideal of IBIS_T is

  quantity Vend1 across Iend1 through n1 to ref1;
  quantity Vend2 across Iend2 through n2 to ref2;

  quantity i12 : current;    -- Current wave traveling from end1 to end2
  quantity i21 : current;    -- Current wave traveling from end2 to end1

  begin

    Iend1 == i12 - i21'delayed(TD);
    Iend2 == i21 - i12'delayed(TD);
    Vend1 == Z0 * (2.0 * i12 - Iend1);
    Vend2 == Z0 * (2.0 * i21 - Iend2);

  end architecture Ideal;
--=============================================================================
--=============================================================================
--=============================================================================
library IEEE;
use IEEE.electrical_systems.all;
use IEEE.math_real.all;

library StatEye_library;
use StatEye_Library.StatEye_functions.all;
--=============================================================================
  entity C_StatEye is

    generic (Cval         : real            := 1.0;
             V0           : real            := 0.0;
             Scale        : real            := 1.0;
             WfmLenght    : real            := 3.0e-9;
             UnitInterval : real            := 200.0e-12;
             Resolution   : real            := 1.0e-3;
             TimeStep     : real            := 1.0e-12;
             BERvalues    : vector_of_reals := (0.0, 0.1, 0.2, 0.3, 0.4, 0.5);
             WriteFile    : boolean         := false);

    port (terminal p, n : electrical);

  end entity C_StatEye;
  --===========================================================================
  architecture ideal of C_StatEye is

  quantity Vout across Iout through p to n;

  begin

    if domain = quiescent_domain use
      Vout == V0;
    else
      Iout == Scale * Cval * Vout'dot;
    end use;

  end architecture ideal;
  --===========================================================================
  --===========================================================================
  architecture StatEye_off of C_StatEye is

  quantity Vout across Iout through p to n;

  signal   CursorIndex : integer := 0;
  signal   EyeIndex    : integer := 1;
  signal   Count       : integer := 1;

  signal   Wfm         : vector_of_reals(1 to integer(floor(WfmLenght/TimeStep))) := (others => 0.0);
  signal   EyeContours : vector_of_real_pairs   (1 to BERvalues'length) := (others => (others => 0.0));
  quantity EyeWfm      : vector_of_voltage_pairs(1 to BERvalues'length) := (others => (others => 0.0));

  begin
    ---------------------------------------------------------------------------
    GetCursorIndex : process is
    begin
      wait for WfmLenght;
      CursorIndex <= 1;
      wait;
    end process GetCursorIndex;

    Ticker : process is
    begin
      wait for TimeStep;
      if (CursorIndex = 0) then
        Wfm(Count) <= Vout;
        Count      <= Count + 1;
      elsif (Now <= WfmLenght+UnitInterval) then
        EyeContours <= (others => (others => 0.0));
        report " Eye slice count: " & integer'image(EyeIndex);
        EyeIndex <= EyeIndex + 1;
      else
        EyeContours <= (others => (others => 0.0));
      end if;
    end process Ticker;
    ---------------------------------------------------------------------------
    break on Count, EyeContours;

    if (domain = quiescent_domain) use
      Vout == V0;
    else
      Iout == Scale * Cval * Vout'dot;
    end use;

    if (now >= WfmLenght) use
      EyeWfm == vector_of_voltage_pairs(EyeContours);
    else
      EyeWfm == (others => (others => 0.0));
    end use;

  end architecture StatEye_off;
--=============================================================================
--=============================================================================
  architecture StatEye_on of C_StatEye is

  quantity Vout across Iout through p to n;

  signal   CursorIndex : integer := 0;
  signal   EyeIndex    : integer := 1;
  signal   Count       : integer := 1;

  signal   Wfm         : vector_of_reals(1 to integer(floor(WfmLenght/TimeStep))) := (others => 0.0);
  signal   EyeContours : vector_of_real_pairs   (1 to BERvalues'length) := (others => (others => 0.0));
  quantity EyeWfm      : vector_of_voltage_pairs(1 to BERvalues'length) := (others => (others => 0.0));

  begin
    ---------------------------------------------------------------------------
    GetCursorIndex : process is
    begin
      wait for WfmLenght;
      CursorIndex <= FindCursorIndex(Wfm);
      wait;
    end process GetCursorIndex;

    Ticker : process is
    begin
      wait for TimeStep;
      if (CursorIndex = 0) then
        Wfm(Count) <= Vout;
        Count      <= Count + 1;
      elsif (Now <= WfmLenght+UnitInterval) then
        EyeContours <= MakeEyeContours(Wfm,
                                       UnitInterval,
                                       CursorIndex,
                                       BERvalues,
                                       EyeIndex,
                                       Resolution,
                                       TimeStep,
                                       WriteFile);
        report " Eye slice count: " & integer'image(EyeIndex);
        EyeIndex <= EyeIndex + 1;
      else
        EyeContours <= (others => (others => 0.0));
      end if;
    end process Ticker;
    ---------------------------------------------------------------------------
    break on Count, EyeContours;
    
    if (domain = quiescent_domain) use
      Vout == V0;
    else
      Iout == Scale * Cval * Vout'dot;
    end use;

    if (now >= WfmLenght) use
      EyeWfm == vector_of_voltage_pairs(EyeContours);
    else
      EyeWfm == (others => (others => 0.0));
    end use;

  end architecture StatEye_on;
  --===========================================================================
  --===========================================================================
